/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : hal_entry
* Version      : 1.0
* Description  : Entry function.
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 30.04.2020 1.00     First Release
***********************************************************************************************************************/

/***********************************************************************************************************************
 Includes
 **********************************************************************************************************************/
#include <stdio.h>
#include "hal_entry.h"
#include "r_sdadc24_control_api.h"
#include "r_usb_control_api.h"
#include "r_ring_buffer_control_api.h"
#include "r_weight_calculate_api.h"

/***********************************************************************************************************************
 Global  functions
 **********************************************************************************************************************/
void R_BSP_WarmStart(bsp_warm_start_event_t event);

/***********************************************************************************************************************
 Typedef definitions
 **********************************************************************************************************************/
/** Weight, A/D value(uncorrect, correct) data structure */
typedef struct
{
    float   weight_uncorrect;                                       /* Measured weight [g] before zero adjustment     */
    float   weight;                                                 /* Measured weight [g]                            */
    float   average_uncorrect;                                      /* SDADC24 average value(offset, gain included)   */
    float   average_correct;                                        /* After offset and gain correction               */
} st_weight_dataset_t;

/** USB transmission request structure */
typedef struct
{
    bool        flag;                                               /* Request flag                                   */
    uint8_t     step;                                               /* Transmission step number                       */
} st_message_request_t;

/** USB transmission request structure */
typedef struct
{
    st_message_request_t    zeroadjust;                             /* Zero adjust message request                    */
    st_message_request_t    weight;                                 /* Weight message request                         */
    st_message_request_t    overflow;                               /* Offset message request                         */
    st_message_request_t    disconnect;                             /* Disconnect detect message request              */
} st_message_control_t;


/***********************************************************************************************************************
 Exported global variables (to be accessed by other files)
 **********************************************************************************************************************/
/** SDADC24 A/D conversion value ring buffer */
volatile uint32_t               g_sdadc24_array[D_ARRAY_LEN];
volatile st_ring_buf_t          g_sdadc24_ring_buf = {
                                                      (uint8_t *) g_sdadc24_array, sizeof (g_sdadc24_array), 0, 0
                                                     };

/***********************************************************************************************************************
 Private global variables and functions
 **********************************************************************************************************************/
/** Variables used USB processing */
static usb_status_t             gs_usb_status;
static usb_setup_t              gs_usb_setup;
static usb_pcdc_linecoding_t    gs_usb_linecoding;
static uint8_t                  gs_usb_readbuf[D_USB_EP_PACKET_SIZE]          = {0U};
static st_usbctrl_ctrl_t        gs_usbctrl_ctrl = {
                                                   &g_basic0_ctrl,      &gs_usb_status, &gs_usb_setup,
                                                   &gs_usb_linecoding   , gs_usb_readbuf, D_USB_EP_PACKET_SIZE
                                                  };

/** Variables used weight measurement */
static st_weight_dataset_t      gs_weight_dataset[D_ARRAY_LEN];         /* Array of weights and ADC value     */
static st_ring_buf_t            gs_weight_ring_buf = {
                                                      (uint8_t *) gs_weight_dataset, sizeof (gs_weight_dataset), 0, 0
                                                     };

/** Variable used SDADC24 processing */
static uint32_t                 gs_sdadc24_ad_array_index = 0U;                 /* Write index for array of average   */
static float                    gs_sdadc24_ad_array[D_WEIGHT_SAMPLE_NUM] = {0U};   /* Array for average calculation   */

/** Variables used message transmission */
/** Zero adjust message */
static char                     gs_zeroadjust_message[D_MESSAGE_STEP_ZEROADJUST][D_USB_EP_PACKET_SIZE] = {
                                                                                                 "[ZeroAdjust], "
};
/** Weight message */

static char                     gs_weight_message[D_MESSAGE_STEP_WEIGHT][D_USB_EP_PACKET_SIZE] = {
                                                                                                 "[Weight], "
};
/** Overflow message */
static char                     gs_overflow_message[D_MESSAGE_STEP_OVERFLOW][D_USB_EP_PACKET_SIZE] = {
                                                                                                 "[Overflow], "
};
/** Disconnect detect message */
static char                     gs_disconnect_message[D_MESSAGE_STEP_DISCONNECT][D_USB_EP_PACKET_SIZE] = {
                                                                                                 "[DisconnectDetect], "
};

/** Request to send message structures */
st_message_control_t            gs_message_control   = {
                                                        {false, 0U},
                                                        {false, 0U},
                                                        {false, 0U},
                                                        {false, 0U},
                                                       };

/***********************************************************************************************************************
 Monitor variable definition (reference only)
***********************************************************************************************************************/
static volatile float             gs_weight = 0.0F;             /* Measured weight [g]                                */
static volatile float             gs_sdadc24_average_uncorrect; /* SDADC24 A/D average value(uncorrected) variable    */
static volatile float             gs_sdadc24_average_correct;   /* SDADC24 A/D average value(corrected) variable      */
static volatile int32_t           gs_sdadc24_value;             /* SDADC24 A/D value storage variable                 */
static volatile int32_t           gs_sdadc24_offset;            /* PGA Offset voltage                                 */
static volatile st_gain_setting_t gs_sdadc24_gain_setting;      /* PGA gain setting structure                         */

/***********************************************************************************************************************
 * Function Name: sw_check
 * Description  : This function determines SW depression.
 * Arguments    : bool * p_sw_status
 *                 Switch status pointer.
 *                 true:Pressed, false:Not pressed
 * Return Value : fsp_err_t
 *                  FSP return value.
 **********************************************************************************************************************/
static fsp_err_t sw_check (bool *p_sw_status);

/***********************************************************************************************************************
 * Function Name: message_transmission
 * Description  : This function message transmission processing.
 * Arguments    : void
 * Return Value : fsp_err_t
 *                  FSP return value.
 **********************************************************************************************************************/
static fsp_err_t message_transmission (void);

/***********************************************************************************************************************
 * Function Name: calc_average
 * Description  : This function takes AD values and returns a arithmetic average or moving average result.
 * Arguments    : int32_t input -
 *                    Input value.
 *              : float p_array[] -
 *                    Previous values array.
 *              : size_t size -
 *                    Number of array elements.
 *              : uint32_t * p_w_index -
 *                    Pointer to write index of array.
 *              : float * p_output -
 *                    Pointer to average value.
 * Return Value : bool
 *                    Calculation state.
 *                    false: Incomplete, true: complete
 **********************************************************************************************************************/
static bool calc_average (float input, float p_array[], size_t size, uint32_t *p_w_index, float *p_output);

/***********************************************************************************************************************
 * Function Name: init_variables
 * Description  : This function initializes variables used for weighing.
 * Arguments    : None
 * Return Value : None
 **********************************************************************************************************************/
static void init_variables (void);

/***********************************************************************************************************************
 * Function Name: hal_entry
 * Description  : The RA Configuration tool generates main() and uses it to generate threads if an RTOS is used.
 *                This function is called by main() when no RTOS is used.
 * Arguments    : None
 * Return Value : None
 **********************************************************************************************************************/
void hal_entry(void)
{
    /** Variables used SDADC24 processing. */
    st_gain_setting_t   sdadc24_gain_setting;                                   /* GSET1, GSET2, GTOTAL structure     */
    int32_t             sdadc24_ad_value;                                       /* SDADC24 A/D conversion value       */
    int32_t             sdadc24_offset;                                         /* PGA Offset voltage                 */
    /** Variables used weight calculation processing. */
    st_weight_dataset_t weight_dataset;                                         /* Weight parameter data set          */
    float               coef_a                      = D_LOADCELL_COEFFICIENT_A; /* Weight conversion coefficient A    */
    float               coef_b                      = D_LOADCELL_COEFFICIENT_B; /* Weight conversion coefficient B    */
    /** Other variables and flags used in application. */
    bool                weight_transmission_enable  = false;                    /* ON/OFF of weight measurement       */
    bool                zero_adjust_request         = false;                    /* Weight zero adjust request flag    */
    bool                disconnect_detect_request   = false;                    /* Disconnect detect request flag     */
    bool                stability_init_request      = false;                    /* Stability check value clear flag   */
    bool                ret;                                                    /* Function return value              */
    volatile fsp_err_t  fsp_err;                                                /* FSP return value                   */

    /****************************************************/
    /** Initialize and start GPT166 (LED1 blink timer)  */
    /****************************************************/
    fsp_err = R_GPT_Open (g_blinker.p_ctrl, g_blinker.p_cfg);

    if (FSP_SUCCESS == fsp_err)
    {
        fsp_err = R_GPT_Start (g_blinker.p_ctrl);
    }
    else
    {
        /** Do nothing */
    }

    /** Handle error. */
    if (FSP_SUCCESS != fsp_err)
    {
        app_error_terminate ();
        APP_ERR_TRAP (fsp_err);
    }
    else
    {
        /** Do nothing. */
    }

    /*****************************/
    /** Initialize and start USB */
    /*****************************/
    /** Open USB instance. */
    fsp_err = R_USB_Open (&g_basic0_ctrl, &g_basic0_cfg);

    /** Handle error. */
    if (FSP_SUCCESS != fsp_err)
    {
        app_error_terminate ();
        APP_ERR_TRAP (fsp_err);
    }
    else
    {
        /** Do nothing. */
    }

    /**************/
    /** Main loop */
    /**************/
    while (true)
    {
        /** Check if switch is pressed. */
    	fsp_err = sw_check (&ret);

        /*************************************************/
        /** Weight measurement enable/disable processing */
        /*************************************************/
    	/** Switch is pressed or disconnect detection process is executed. */
    	if (FSP_SUCCESS == fsp_err)
    	{
            if (true == ret)
            {
                /** Weight measurement disabled state. */
                if (false == weight_transmission_enable)
                {
                    /** Initialize application variables. */
                    init_variables ();
                    weight_transmission_enable  = true;
                    zero_adjust_request         = true;
                    stability_init_request      = true;
                    R_SDADC24_Init (&g_sdadc24_cfg, &g_sdadc24_channel_cfg, &g_sdadc24_channel_cfgs,
                                    &g_sdadc24_extend_cfg);

                    /** Set offset voltage and start continuous scan. */
                    fsp_err = R_SDADC_Open (&g_sdadc24_ctrl, &g_sdadc24_cfg);

                    if (FSP_SUCCESS == fsp_err)
                    {
                        fsp_err = R_SDADC24_ScanStart (&g_sdadc24_ctrl, D_SDADC24_OFFSET);
                    }
                    else
                    {
                        /** Do nothing. */
                    }

                    /** Stop the LED1 blinking timer(GPT) and turn on LED1. */
                    if (FSP_SUCCESS == fsp_err)
                    {
                        fsp_err = R_GPT_Stop (g_blinker.p_ctrl);
                        TURN_RED_ON
                    }
                    else
                    {
                        /** Do nothing. */
                    }
                }
                /** Weight measurement enabled state. */
                else
                {
                    weight_transmission_enable = false;

                    /** Stop continuous scan. */
                    fsp_err = R_SDADC_ScanStop (&g_sdadc24_ctrl);
                    if (FSP_SUCCESS == fsp_err)
                    {
                        fsp_err = R_SDADC_Close (&g_sdadc24_ctrl);
                    }
                    else
                    {
                        /** Do nothing. */
                    }

                    /** LED1 starts blinking. */
                    if (FSP_SUCCESS == fsp_err)
                    {
                        fsp_err = R_GPT_Start (g_blinker.p_ctrl);
                    }
                    else
                    {
                      /** Do nothing. */
                    }
                }
    		}
            else
            {
                /** Do nothing. */
            }
    	}
    	else
    	{
    	    /** Do nothing. */
    	}

        /** Handle error. */
        if (FSP_SUCCESS != fsp_err)
        {
            app_error_terminate ();
            APP_ERR_TRAP (fsp_err);
        }
        else
        {
            /** Do nothing. */
        }

        /**********************************/
        /** Weight measurement processing */
        /**********************************/
    	/** A/D conversion complete for SDADC? */
        if (0U != R_RINGBUF_GetData ((st_ring_buf_t *) &g_sdadc24_ring_buf, (uint8_t *) &sdadc24_ad_value,
                                     sizeof (sdadc24_ad_value),             D_UPDATE))
    	{
            /** A/D value does not over flow. */
    		if (D_SDADC24_OVERFLOW_THRESH > sdadc24_ad_value)
    		{
    			/** Sign extension. */
    			sdadc24_ad_value = (sdadc24_ad_value << 8U) >> 8U;

    			/** Calculate average of loadcell A/D conversion value[LSB]. */
                ret = calc_average ((float) sdadc24_ad_value,          gs_sdadc24_ad_array,
                                    (size_t) D_WEIGHT_SAMPLE_NUM,      &gs_sdadc24_ad_array_index,
                                    &weight_dataset.average_uncorrect);

                /** Computation of the average value of A/D values for D_WEIGHT_SAMPLE_NUM times is completed. */
                if (true == ret)
                {
                    /** Calculate weight [g] from A/D conversion value[LSB] of loadcell. */
                    weight_dataset.weight_uncorrect = R_WEIGHT_Calculate (weight_dataset.average_uncorrect, coef_a, coef_b);

                    /*******************************/
                    /** Zero adjustment processing */
                    /*******************************/
                    if (zero_adjust_request == true)
                    {
                        /** If the weight measurement value is stable, set it to zero grams. */
                        if (R_WEIGHT_StabilityCheck (weight_dataset.weight, &stability_init_request))
                        {
                            R_WEIGHT_SetZeroAdjustWeight (weight_dataset.weight);
                            zero_adjust_request = false;
                            sprintf ((char *)&gs_zeroadjust_message[1], "Zero adjust value is %.1f[g].\r\n",
                                     weight_dataset.weight);
                        }
                        else
                        {
                            sprintf ((char *)&gs_zeroadjust_message[1], "The measured value is not stable.\r\n");
                        }
                        gs_message_control.zeroadjust.step = 0U;
                        gs_message_control.zeroadjust.flag = true;
                    }
                    else
                    {
                        /** Do nothing. */
                    }

                    /** Calculate corrected weight from measured weight and zero weight. */
                    weight_dataset.weight = (weight_dataset.weight_uncorrect - R_WEIGHT_GetZeroAdjustWeight());

                    /** Store weight data in ring buffer. */
                    R_RINGBUF_SetData ((st_ring_buf_t *) &gs_weight_ring_buf,
                                       (uint8_t *) &weight_dataset, sizeof (weight_dataset));

                    /** If the weight is out of the measuring range, perform disconnect detection.
                     *  This judgment includes the weight of zero adjustment.
                     */
                    if ((D_LOADCELL_MIN_OVERLOAD > weight_dataset.weight_uncorrect) ||
                        (D_LOADCELL_MAX_OVERLOAD < weight_dataset.weight_uncorrect))
                    {
                        disconnect_detect_request = true;
                        sprintf ((char *) &gs_overflow_message[1], "Weight overload.\r\n");
                        gs_message_control.overflow.step = 0U;
                        gs_message_control.overflow.flag = true;
                    }
                    else
                    {
                        /** Do nothing. */
                    }

                    /** Debug monitor. */
                    {
                        gs_sdadc24_value                = sdadc24_ad_value;
                        gs_sdadc24_average_uncorrect    = weight_dataset.average_uncorrect;
                        gs_sdadc24_average_correct      = weight_dataset.average_correct;
                        gs_sdadc24_offset               = R_SDADC24_GetOffset (D_SDADC24_WEIGHT_CHANNEL);
                        R_SDADC24_GetGain (&sdadc24_gain_setting, D_SDADC24_WEIGHT_CHANNEL);
                        gs_sdadc24_gain_setting.gset1   = sdadc24_gain_setting.gset1;
                        gs_sdadc24_gain_setting.gset2   = sdadc24_gain_setting.gset2;
                        gs_sdadc24_gain_setting.gtotal  = sdadc24_gain_setting.gtotal;
                        gs_weight                       = weight_dataset.weight;
                    }
                }
                else
                {
                    /** Do nothing. */
                }
    		}
    		/** A/D value overflow. */
    		else
    		{
    		    disconnect_detect_request = true;
    		    sprintf ((char *) &gs_overflow_message[1], "A/D conversion value overflow.\r\n");
                gs_message_control.overflow.step = 0U;
                gs_message_control.overflow.flag = true;
    		}
    	}
        else
        {
            /** Do nothing. */
        }

        /************************************/
        /** Disconnect detection processing */
        /************************************/
        if (true == disconnect_detect_request)
        {
            disconnect_detect_request = false;

            /** Stop continuous scan. */
            fsp_err = R_SDADC_ScanStop (&g_sdadc24_ctrl);
            if (FSP_SUCCESS == fsp_err)
            {
                fsp_err = R_SDADC_Close (&g_sdadc24_ctrl);
            }
            else
            {
                /** Do nothing. */
            }

            if (FSP_SUCCESS == fsp_err)
            {
                /** Change the SDADC24 setting for disconnection detection and
                 *  execute disconnection detection processing.
                 */
                fsp_err = R_SDADC24_DisconnectDetect (&g_sdadc24_ctrl,           &g_sdadc24_cfg,
                                                      &g_sdadc24_channel_cfg,    &g_sdadc24_channel_cfgs,
                                                      &g_sdadc24_extend_cfg,     &ret);

                if (FSP_SUCCESS == fsp_err)
                {
                    /** Connected, continue weighing process. */
                    if (true == ret)
                    {
                        sprintf ((char *) &gs_disconnect_message[1], "Connected - continue weighing process.\r\n");

                        /** Initialize application variables. */
                        init_variables ();
                        R_SDADC24_Init(&g_sdadc24_cfg,          &g_sdadc24_channel_cfg,
                                       &g_sdadc24_channel_cfgs, &g_sdadc24_extend_cfg);

                        /** Set offset voltage and start continuous scan. */
                        fsp_err = R_SDADC_Open (&g_sdadc24_ctrl, &g_sdadc24_cfg);
                        if (FSP_SUCCESS == fsp_err)
                        {
                            fsp_err = R_SDADC24_ScanStart (&g_sdadc24_ctrl, D_SDADC24_OFFSET);
                        }
                        else
                        {
                            /** Do nothing. */
                        }
                    }
                    /** Disconnected, stop weighing process. */
                    else
                    {
                        sprintf ((char *) &gs_disconnect_message[1], "Disconnected - stop weighing process.\r\n");

                        /** LED1 blinks and transitions to the weight measurement disable. */
                        fsp_err = R_GPT_Start (g_blinker.p_ctrl);
                        weight_transmission_enable = false;
                    }
                    gs_message_control.disconnect.step = 0U;
                    gs_message_control.disconnect.flag = true;
                }
                else
                {
                    /** Do nothing */
                }
            }
            else
            {
              /** Do nothing. */
            }

            /** Handle error. */
            if (FSP_SUCCESS != fsp_err)
            {
                app_error_terminate ();
                APP_ERR_TRAP (fsp_err);
            }
            else
            {
                /** Do nothing. */
            }
        }
        else
        {
            /** Do nothing. */
        }

        /************************************/
        /** Message transmission processing */
        /************************************/
        /** Weight measurement enabled state. */
        if (true == weight_transmission_enable)
        {
            if (false == gs_message_control.weight.flag)
            {
                /** Get weight data from ring buffer. */
                if (0U != R_RINGBUF_GetData ((st_ring_buf_t *) &gs_weight_ring_buf,
                                             (uint8_t *) &weight_dataset,
                                             sizeof (weight_dataset),
                                             D_UPDATE))
                {
                    /** Get current Offset and gain setting. */
                    sdadc24_offset = R_SDADC24_GetOffset (D_SDADC24_WEIGHT_CHANNEL);
                    R_SDADC24_GetGain (&sdadc24_gain_setting, D_SDADC24_WEIGHT_CHANNEL);
                    /** Offset and gain correction. (For sending messages) */
                    weight_dataset.average_correct = R_WEIGHT_ADValueCorrect (weight_dataset.average_uncorrect,
                                                                              &sdadc24_gain_setting, sdadc24_offset);
                    /** Calculate offset value. */
                    sdadc24_offset = (sdadc24_offset * sdadc24_gain_setting.gset2);
                    /** Create send message and set transmission request flag. */
                    sprintf ((char *) &gs_weight_message[1], "%.1f, %.1f, %f, %f, ",
                             weight_dataset.weight,
                             R_WEIGHT_GetZeroAdjustWeight(),
                             weight_dataset.average_uncorrect,
                             weight_dataset.average_correct);
                    sprintf ((char *) &gs_weight_message[2], "%ld, %u, %u, %u\r\n",
                             sdadc24_offset,
                             sdadc24_gain_setting.gset1,
                             sdadc24_gain_setting.gset2,
                             sdadc24_gain_setting.gtotal);
                    gs_message_control.weight.step = 0U;
                    gs_message_control.weight.flag = true;
                }
                else
                {
                    /** Do nothing. */
                }
            }
            else
            {
                /** Do nothing. */
            }
        }
        else
        {
            /** Do nothing. */
        }

        /** USB write completed. */
        if (true == R_USBCTRL_GetWriteStatus ())
        {
            fsp_err = message_transmission ();

            /** Handle error. */
            if (FSP_SUCCESS != fsp_err)
            {
                app_error_terminate ();
                APP_ERR_TRAP (fsp_err);
            }
            else
            {
                /** Do nothing. */
            }
        }
        else
        {
            /** Do nothing. */
        }

        /**********************************/
        /** USB communication processing  */
        /**********************************/
        fsp_err = R_USBCTRL_EventPoll (&gs_usbctrl_ctrl);

        /** Handle error. */
        if (FSP_SUCCESS != fsp_err)
        {
            app_error_terminate ();
            APP_ERR_TRAP (fsp_err);
        }
        else
        {
            /** Do nothing. */
        }
    }
}
/***********************************************************************************************************************
 End of function hal_entry
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_BSP_WarmStart
 * Description  : This function is called at various points during the startup process.
 *                This implementation uses the event that is called right before main() to set up the pins.
 * Arguments    : bsp_warm_start_event_t event
 *                  Where at in the start up process the code is currently at
 * Return Value : void
 **********************************************************************************************************************/
void R_BSP_WarmStart(bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_POST_C == event)
    {
        /** C runtime environment and system clocks are setup. */
        /** Configure pins. */
        R_IOPORT_Open (&g_ioport_ctrl, &g_bsp_pin_cfg);
    }
    else
    {
        /** Do nothing. */
    }
}
/***********************************************************************************************************************
 End of function R_BSP_WarmStart
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: sw_check
 * Description  : This function determines SW depression.
 * Arguments    : bool * p_sw_status
 *                 Switch status pointer.
 *                 true:Pressed, false:Not pressed
 * Return Value : fsp_err_t
 *                  FSP return value.
 **********************************************************************************************************************/
static fsp_err_t sw_check (bool *p_sw_status)
{
    bool            sw_input        = false;
    static bool     s_sw_last       = false;
    static bool     s_sw_last_fix   = false;
    bsp_io_level_t  io_level        = 0U;
    fsp_err_t       fsp_err;
    *p_sw_status                    = false;

    /** Switch input check. */
    fsp_err = R_IOPORT_PinRead (&g_ioport_ctrl, BSP_IO_PORT_02_PIN_06, &io_level);

    if (FSP_SUCCESS == fsp_err)
    {
        if (BSP_IO_LEVEL_LOW == io_level)
        {
            /** Switch push state. */
            sw_input = true;
        }
        else
        {
            /** Do nothing. */
        }

        /** Switch pressing judgment. */
        if (sw_input == s_sw_last)
        {
            if ((false == s_sw_last_fix) && (true == sw_input))
            {
                /** Switch press detection. */
                *p_sw_status = true;
            }
            else
            {
                /** Do nothing. */
            }
            s_sw_last_fix = sw_input;
        }
        else
        {
            /** Do nothing. */
        }

        s_sw_last = sw_input;
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function sw_check
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: message_transmission
 * Description  : This function message transmission processing.
 * Arguments    : void
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
static fsp_err_t message_transmission (void)
{
    uint32_t                message_length      = 0U;
    st_message_request_t    *p_message_request  = NULL;
    uint8_t                 *p_message          = NULL;
    uint8_t                 step_max            = 0U;
    fsp_err_t               fsp_err             = FSP_SUCCESS;

    if (true == gs_message_control.zeroadjust.flag)
    {
        p_message_request   = &gs_message_control.zeroadjust;
        p_message           = (uint8_t*) &gs_zeroadjust_message[p_message_request->step];
        message_length      = (uint32_t) strlen ((char *) &gs_zeroadjust_message[p_message_request->step]);
        step_max            = D_MESSAGE_STEP_ZEROADJUST;
    }
    else if (true == gs_message_control.weight.flag)
    {
        p_message_request   = &gs_message_control.weight;
        p_message           = (uint8_t*) &gs_weight_message[p_message_request->step];
        message_length      = (uint32_t) strlen ((char *) &gs_weight_message[p_message_request->step]);
        step_max            = D_MESSAGE_STEP_WEIGHT;
    }
    else if (true == gs_message_control.overflow.flag)
    {
        p_message_request   = &gs_message_control.overflow;
        p_message           = (uint8_t*) &gs_overflow_message[p_message_request->step];
        message_length      = (uint32_t) strlen ((char *) &gs_overflow_message[p_message_request->step]);
        step_max            = D_MESSAGE_STEP_OVERFLOW;
    }
    else if (true == gs_message_control.disconnect.flag)
    {
        p_message_request   = &gs_message_control.disconnect;
        p_message           = (uint8_t*) &gs_disconnect_message[p_message_request->step];
        message_length      = (uint32_t) strlen ((char *) &gs_disconnect_message[p_message_request->step]);
        step_max            = D_MESSAGE_STEP_DISCONNECT;
    }
    else
    {
        /** Do nothing. */
    }

    if (NULL != p_message)
    {
        /** Start sending message. */
        fsp_err = R_USBCTRL_Write (&gs_usbctrl_ctrl, p_message, message_length, USB_CLASS_PCDC);

        if (FSP_SUCCESS == fsp_err)
        {
            p_message_request->step++;
            /** Clear the transmission request flag when transmission is completed. */
            if (step_max <= p_message_request->step)
            {
                p_message_request->flag = false;
            }
            else
            {
                /** Do nothing. */
            }
        }
        else
        {
            /** Do nothing. */
        }
    }
    else
    {
        /** Do nothing. */
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function message_transmission
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: calc_average
 * Description  : This function takes AD values and returns a arithmetic average or moving average result.
 * Arguments    : int32_t input -
 *                    Input value.
 *              : float p_array[] -
 *                    Previous values array.
 *              : size_t size -
 *                    Number of array elements.
 *              : uint32_t * p_w_index -
 *                    Pointer to write index of array.
 *              : float * p_output -
 *                    Pointer to average value.
 * Return Value : bool
 *                    Calculation state.
 *                    false: Incomplete, true: complete
 **********************************************************************************************************************/
static bool calc_average (float input, float p_array[], size_t size, uint32_t *p_w_index, float *p_output)
{
    size_t      i;
    float       result  = 0.0F;
    uint32_t    index   = *p_w_index;
    bool        ret     = true;

    /** Data update. */
    p_array[(index % size)] = input;
    index++;

    if (index % size != 0)
    {
        ret = false;
    }
    else
    {
        index = 0;
    }

    *p_w_index = index;

    if (true == ret)
    {
        /** Calculation of total value. */
        for (i = 0U; i < size; i++)
        {
            result += p_array[i];
        }
        /** Averaging. */
        *p_output = (result / (float) size);
    }
    else
    {
        /** Do nothing. */
    }

    return ret;
}
/***********************************************************************************************************************
 End of function calc_average
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: init_variables
 * Description  : This function initializes variables used for weighing.
 * Arguments    : None
 * Return Value : None
 **********************************************************************************************************************/
static void init_variables (void)
{
    gs_sdadc24_ad_array_index = 0U;
    (void) R_RINGBUF_SetDataIndex (&gs_weight_ring_buf, 0U, D_R_INDEX);
    (void) R_RINGBUF_SetDataIndex (&gs_weight_ring_buf, 0U, D_W_INDEX);
    R_WEIGHT_SetZeroAdjustWeight (0.0F);
}
/***********************************************************************************************************************
 End of function init_variables
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: app_error_terminate
 * Description  : This function termination process when an error occurs.
 * Arguments    : None
 * Return Value : None
 **********************************************************************************************************************/
void app_error_terminate (void)
{
    (void) R_USB_Close (&g_basic0_ctrl);
    (void) R_SDADC_ScanStop (&g_sdadc24_ctrl);
    (void) R_SDADC_Close (&g_sdadc24_ctrl);
    (void) R_GPT_Stop (g_blinker.p_ctrl);
    (void) R_GPT_Close (g_blinker.p_ctrl);
    TURN_RED_OFF
}
/***********************************************************************************************************************
 End of function app_error_terminate
 **********************************************************************************************************************/

