/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_interrupt_callback.c
* Version      : 1.0
* Description  : Interrupt callback processing
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 30.04.2020 1.00     First Release
***********************************************************************************************************************/

/***********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 **********************************************************************************************************************/
#include "board_cfg.h"
#include "hal_entry.h"
#include "r_sdadc24_control_api.h"
#include "r_usb_control_api.h"
#include "r_ring_buffer_control_api.h"
#include "r_interrupt_callback.h"

/***********************************************************************************************************************
 Typedef definitions
 **********************************************************************************************************************/

/***********************************************************************************************************************
 Exported global variables
 **********************************************************************************************************************/
extern volatile st_ring_buf_t  g_sdadc24_ring_buf;

/***********************************************************************************************************************
 Global variables and functions
 **********************************************************************************************************************/
uint32_t g_sdadc24_ad_value;

/***********************************************************************************************************************
 Private (static) variables and functions
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_conversion_callback
 * Description  : This function SDADC24 conversion completion interrupt callback function.
 * Arguments    : adc_callback_args_t *p_args
 *                  ADC callback arguments structure pointer.
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_conversion_callback(adc_callback_args_t *p_args)
{
    uint32_t    sdadc24_ad_value;
    uint32_t    overflow;
    fsp_err_t   fsp_err;

    if (p_args->event == ADC_EVENT_CONVERSION_COMPLETE)
    {
        fsp_err = R_SDADC_Read32 (&g_sdadc24_ctrl, p_args->channel, &sdadc24_ad_value);

        if (FSP_SUCCESS == fsp_err)
        {
            /** Check for SDADC24 overflow. */
            overflow = R_SDADC0->ADCR_b.SDADCRS;
            if (overflow)
            {
                sdadc24_ad_value |= R_SDADC0_ADCR_SDADCRS_Msk;
            }
            else
            {
                /** Do nothing */
            }

            if (0U == R_RINGBUF_SetData ((st_ring_buf_t *) &g_sdadc24_ring_buf, (uint8_t *) &sdadc24_ad_value,
                                         sizeof (sdadc24_ad_value)))
            {
                /** Ring buffer error, Turn OFF RED LED to indicate fatal error */
                app_error_terminate ();
                APP_ERR_TRAP (fsp_err);
            }
        }
        else
        {
            /** Handle error, Turn OFF RED LED to indicate fatal error */
            app_error_terminate ();
            APP_ERR_TRAP (fsp_err);
        }
    }
    else
    {
        /** Do nothing. */
    }
}
/***********************************************************************************************************************
 End of function R_SDADC24_conversion_callback
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_conversion_oneshot callback
 * Description  : This function SDADC24 conversion completion interrupt callback function.
 * Arguments    : adc_callback_args_t *p_args
 *                  ADC callback arguments structure pointer.
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_conversion_oneshot_callback(adc_callback_args_t *p_args)
{
    uint32_t    overflow;
    fsp_err_t   fsp_err;

    if (p_args->event == ADC_EVENT_CONVERSION_COMPLETE)
    {
        fsp_err = R_SDADC_Read32 (&g_sdadc24_ctrl, p_args->channel, &g_sdadc24_ad_value);

        if (FSP_SUCCESS == fsp_err)
        {
            /** Check for SDADC24 overflow. */
            overflow = R_SDADC0->ADCR_b.SDADCRS;
            if (overflow)
            {
                g_sdadc24_ad_value |= R_SDADC0_ADCR_SDADCRS_Msk;
            }
            else
            {
                /** Do nothing. */
            }
        }
        else
        {
            /** Handle error, Turn OFF RED LED to indicate fatal error */
            app_error_terminate ();
            APP_ERR_TRAP (fsp_err);
        }
    }
    else
    {
        /** Do nothing. */
    }
}
/***********************************************************************************************************************
 End of function R_SDADC24_conversion_oneshot_callback
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_GPT166_blinker_callback
 * Description  : This function GPT166 overflow interrupt callback function.
 * Arguments    : timer_callback_args_t *p_args
 *                  Timer callback arguments structure pointer.
 * Return Value : None
 **********************************************************************************************************************/
void R_GPT166_blinker_callback(timer_callback_args_t *p_args)
{
    fsp_err_t       fsp_err;
    bsp_io_level_t  io_level;

    FSP_PARAMETER_NOT_USED(p_args);
    fsp_err = R_IOPORT_PinRead (&g_ioport_ctrl, g_bsp_leds.p_leds[RED], &io_level);

    if (FSP_SUCCESS == fsp_err)
    {
        if (ON == io_level)
        {
            TURN_RED_OFF
        }
        else
        {
            TURN_RED_ON
        }
    }
    else
    {
        /** Handle error, Turn OFF RED LED to indicate fatal error */
        app_error_terminate ();
        APP_ERR_TRAP (fsp_err);
    }
}
/***********************************************************************************************************************
 End of function R_GPT166_blinker_callback
 **********************************************************************************************************************/
