/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_sdadc24_control_api.c
* Version      : 1.0
* Description  : SDADC24 control module
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 30.04.2020 1.00     First Release
***********************************************************************************************************************/

/***********************************************************************************************************************
 Includes <System Includes> , “Project Includes”
 **********************************************************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "board_cfg.h"
#include "hal_data.h"
#include "r_sdadc24_control_api.h"
#include "r_interrupt_callback.h"

/***********************************************************************************************************************
 Macro definitions
 **********************************************************************************************************************/

/***********************************************************************************************************************
 Local Typedef definitions
 **********************************************************************************************************************/

/***********************************************************************************************************************
 Exported global variables (to be accessed by other files)
 **********************************************************************************************************************/
sdadc_instance_ctrl_t               g_sdadc24_ctrl;
sdadc_scan_cfg_t                    g_sdadc24_channel_cfg;
sdadc_channel_cfg_t                 g_sdadc24_channel_cfgs = {
                                                              .stage_2_gain     = SDADC_CHANNEL_STAGE_2_GAIN_8,
                                                              .stage_1_gain     = SDADC_CHANNEL_STAGE_1_GAIN_4,
                                                              .oversampling     = SDADC_CHANNEL_OVERSAMPLING_2048,
                                                              .polarity         = SDADC_CHANNEL_POLARITY_POSITIVE,
                                                              .input            = SDADC_CHANNEL_INPUT_DIFFERENTIAL,
                                                              .average          = SDADC_CHANNEL_AVERAGE_NONE,
                                                              .invert           = SDADC_CHANNEL_INVERSION_OFF,
                                                              .coefficient_m    = 1,
                                                              .coefficient_n    = 0,
                                                              .count_formula    = SDADC_CHANNEL_COUNT_FORMULA_LINEAR,
                                                            };
sdadc_extended_cfg_t                g_sdadc24_extend_cfg  = {
#if defined(VECTOR_NUMBER_SDADC0_ADI)
                                                              .conv_end_irq     = VECTOR_NUMBER_SDADC0_ADI,
#else
                                                              .conv_end_irq     = FSP_INVALID_VECTOR,
#endif
                                                              .conv_end_ipl     = (2),
                                                              .vref_src         = SDADC_VREF_SRC_INTERNAL,
                                                              .vref_voltage     = SDADC_VREF_VOLTAGE_2000_MV,
                                                              .p_channel_cfgs[0] = NULL,
                                                              .p_channel_cfgs[1] = NULL,
                                                              .p_channel_cfgs[2] = NULL,
                                                              .p_channel_cfgs[3] = NULL,
                                                              .p_channel_cfgs[4] = NULL,
                                                            };
adc_cfg_t                           g_sdadc24_cfg         = {
                                                              .unit              = 0,
                                                              .mode              = ADC_MODE_CONTINUOUS_SCAN,
                                                              .resolution        = ADC_RESOLUTION_24_BIT,
                                                              .alignment         = ADC_ALIGNMENT_RIGHT,
                                                              .trigger           = ADC_TRIGGER_SOFTWARE,
#if defined(VECTOR_NUMBER_SDADC0_SCANEND)
                                                              .scan_end_irq      = VECTOR_NUMBER_SDADC0_SCANEND,
#else
                                                              .scan_end_irq      = FSP_INVALID_VECTOR,
#endif
                                                              .scan_end_ipl      = (BSP_IRQ_DISABLED),
#if defined(VECTOR_NUMBER_SDADC0_CALIEND)
                                                              .scan_end_b_irq    = VECTOR_NUMBER_SDADC0_CALIEND,
#else
                                                              .scan_end_b_irq    = FSP_INVALID_VECTOR,
#endif
                                                              .scan_end_b_ipl    = (2),
                                                              .p_callback        = R_SDADC24_conversion_callback,
                                                              .p_context         = NULL,
                                                              .p_extend          = &g_sdadc24_extend_cfg,
                                                            };
/* Instance structure to use this module. */
adc_instance_t                      g_sdadc24             = {
                                                              .p_ctrl           = &g_sdadc24_ctrl,
                                                              .p_cfg            = &g_sdadc24_cfg,
                                                              .p_channel_cfg    = &g_sdadc24_channel_cfg,
                                                              .p_api            = &g_adc_on_sdadc
                                                            };

/***********************************************************************************************************************
 Private (static) variables and functions
 **********************************************************************************************************************/
static const uint8_t gs_gain_list[][3] =
{
/**  GSET1, GSET2, GTOTAL */
     {1,  1,  1},                   /* E_PGA_SDADC24_GAIN_1_1_1     */
     {1,  2,  2},                   /* E_PGA_SDADC24_GAIN_1_2_2     */
     {1,  4,  4},                   /* E_PGA_SDADC24_GAIN_1_4_4     */
     {1,  8,  8},                   /* E_PGA_SDADC24_GAIN_1_8_8     */
     {2,  1,  2},                   /* E_PGA_SDADC24_GAIN_2_1_2     */
     {2,  2,  4},                   /* E_PGA_SDADC24_GAIN_2_2_4     */
     {2,  4,  8},                   /* E_PGA_SDADC24_GAIN_2_4_8     */
     {2,  8, 16},                   /* E_PGA_SDADC24_GAIN_2_8_16    */
     {3,  1,  3},                   /* E_PGA_SDADC24_GAIN_3_1_3     */
     {3,  2,  6},                   /* E_PGA_SDADC24_GAIN_3_2_6     */
     {3,  4, 12},                   /* E_PGA_SDADC24_GAIN_3_4_12    */
     {3,  8, 24},                   /* E_PGA_SDADC24_GAIN_3_8_24    */
     {4,  1,  4},                   /* E_PGA_SDADC24_GAIN_4_1_4     */
     {4,  2,  8},                   /* E_PGA_SDADC24_GAIN_4_2_8     */
     {4,  4, 16},                   /* E_PGA_SDADC24_GAIN_4_4_16    */
     {4,  8, 32},                   /* E_PGA_SDADC24_GAIN_4_8_32    */
     {8,  1,  8},                   /* E_PGA_SDADC24_GAIN_8_1_8     */
     {8,  2, 16},                   /* E_PGA_SDADC24_GAIN_8_2_16    */
     {8,  4, 32}                    /* E_PGA_SDADC24_GAIN_8_4_32    */
};

static const int32_t gs_offset_list[] =
{
            0,                      /* E_PGA_SDADC24_OFFSET_0P00    */
       114688,                      /* E_PGA_SDADC24_OFFSET_10P94   */
       229376,                      /* E_PGA_SDADC24_OFFSET_21P88   */
       344064,                      /* E_PGA_SDADC24_OFFSET_32P81   */
       458752,                      /* E_PGA_SDADC24_OFFSET_43P75   */
       573440,                      /* E_PGA_SDADC24_OFFSET_54P69   */
       688128,                      /* E_PGA_SDADC24_OFFSET_65P63   */
       802816,                      /* E_PGA_SDADC24_OFFSET_76P56   */
       917504,                      /* E_PGA_SDADC24_OFFSET_87P50   */
      1032192,                      /* E_PGA_SDADC24_OFFSET_98P44   */
      1148880,                      /* E_PGA_SDADC24_OFFSET_109P38  */
      1261568,                      /* E_PGA_SDADC24_OFFSET_120P31  */
      1376256,                      /* E_PGA_SDADC24_OFFSET_131P25  */
      1490944,                      /* E_PGA_SDADC24_OFFSET_142P19  */
      1605632,                      /* E_PGA_SDADC24_OFFSET_153P13  */
      1720320,                      /* E_PGA_SDADC24_OFFSET_164P06  */
            0,                      /* Setting prohibited, dummy    */
     -1720320,                      /* E_PGA_SDADC24_OFFSET_M164P06 */
     -1605632,                      /* E_PGA_SDADC24_OFFSET_M153P13 */
     -1490944,                      /* E_PGA_SDADC24_OFFSET_M142P19 */
     -1376256,                      /* E_PGA_SDADC24_OFFSET_M131P25 */
     -1261568,                      /* E_PGA_SDADC24_OFFSET_M120P31 */
     -1148880,                      /* E_PGA_SDADC24_OFFSET_M109P38 */
     -1032192,                      /* E_PGA_SDADC24_OFFSET_M98P44  */
      -917504,                      /* E_PGA_SDADC24_OFFSET_M87P50  */
      -802816,                      /* E_PGA_SDADC24_OFFSET_M76P56  */
      -688128,                      /* E_PGA_SDADC24_OFFSET_M65P63  */
      -573440,                      /* E_PGA_SDADC24_OFFSET_M54P69  */
      -458752,                      /* E_PGA_SDADC24_OFFSET_M43P75  */
      -344064,                      /* E_PGA_SDADC24_OFFSET_M32P81  */
      -229376,                      /* E_PGA_SDADC24_OFFSET_M21P88  */
      -114688,                      /* E_PGA_SDADC24_OFFSET_M10P94  */
};

/***********************************************************************************************************************
 * Function Name: r_sdadc24_oneshot
 * Description  : This function A/D conversion is executed once.
 *                The A/D conversion mode of SDADC24 must be single scan.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                int32_t *p_ad_value
 *                  A/D conversion value storage area pointer.
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
static fsp_err_t r_sdadc24_oneshot (adc_ctrl_t *p_instance, int32_t *p_ad_value);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_Init
 * Description  : This function set a value to the variables of SDADC24.
 * Arguments    : adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 *                sdadc_scan_cfg_t *p_channel_cfg
 *                  ADC scan configuration structure pointer.
 *                sdadc_channel_cfg_t *p_channel_cfgs
 *                  ADC channel configuration structure pointer.
 *                sdadc_extended_cfg_t *p_extended_cfg
 *                  ADC extended configuration structure pointer.
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_Init (adc_cfg_t *p_cfg,                    sdadc_scan_cfg_t *p_channel_cfg,
                     sdadc_channel_cfg_t *p_channel_cfgs, sdadc_extended_cfg_t *p_extended_cfg)
{
    uint8_t                         i;
    sdadc_channel_stage_1_gain_t    stage_1_gain;
    sdadc_channel_stage_2_gain_t    stage_2_gain;

    stage_1_gain = ((D_SDADC24_GAIN & 0b00011100) >> 2);
    stage_2_gain = (D_SDADC24_GAIN & 0b00000011);

    p_cfg->mode                     = ADC_MODE_CONTINUOUS_SCAN;
    p_cfg->p_callback               = R_SDADC24_conversion_callback;
    p_channel_cfg->scan_mask        = 1U << D_SDADC24_WEIGHT_CHANNEL;
    p_channel_cfgs->stage_1_gain    = stage_1_gain;
    p_channel_cfgs->stage_2_gain    = stage_2_gain;
    p_channel_cfgs->average         = SDADC_CHANNEL_AVERAGE_NONE;
    p_channel_cfgs->input           = SDADC_CHANNEL_INPUT_DIFFERENTIAL;
    p_channel_cfgs->oversampling    = SDADC_CHANNEL_OVERSAMPLING_2048;

    /** Continuous scan setting for weight measurement. */
    for (i = ADC_CHANNEL_0; i <= ADC_CHANNEL_4; i++)
    {
        if (D_SDADC24_WEIGHT_CHANNEL == i)
        {
            p_extended_cfg->p_channel_cfgs[i]   = &g_sdadc24_channel_cfgs;
        }
        else
        {
            p_extended_cfg->p_channel_cfgs[i]   = NULL;
        }
    }
}
/***********************************************************************************************************************
 End of function R_SDADC24_Init
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_ConfigDisconnectDetection
 * Description  : This function set a value to the variables of SDADC24.
 * Arguments    : adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 *                sdadc_scan_cfg_t *p_channel_cfg
 *                  ADC scan configuration structure pointer.
 *                sdadc_channel_cfg_t *p_channel_cfgs
 *                  ADC channel configuration structure pointer.
 *                sdadc_extended_cfg_t *p_extended_cfg
 *                  ADC extended configuration structure pointer.
 *                uint8_t polarity
 *                  ADC channel polarity. (0: Positive, 1: Negative)
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_ConfigDisconnectDetection (adc_cfg_t *p_cfg,                    sdadc_scan_cfg_t *p_channel_cfg,
                                          sdadc_channel_cfg_t *p_channel_cfgs, sdadc_extended_cfg_t *p_extended_cfg,
                                          uint8_t polarity)
{
    uint8_t i;

    p_cfg->mode                     = ADC_MODE_SINGLE_SCAN;
    p_cfg->p_callback               = R_SDADC24_conversion_oneshot_callback;
    p_channel_cfg->scan_mask        = 1U << D_SDADC24_WEIGHT_CHANNEL;
    p_channel_cfgs->stage_1_gain    = SDADC_CHANNEL_STAGE_1_GAIN_1;
    p_channel_cfgs->stage_2_gain    = SDADC_CHANNEL_STAGE_2_GAIN_1;
    p_channel_cfgs->average         = SDADC_CHANNEL_AVERAGE_NONE;
    p_channel_cfgs->input           = SDADC_CHANNEL_INPUT_SINGLE_ENDED;
    p_channel_cfgs->oversampling    = SDADC_CHANNEL_OVERSAMPLING_2048;
    p_channel_cfgs->polarity        = (sdadc_channel_polarity_t) polarity;

    /** Continuous scan setting for weight measurement. */
    for (i = ADC_CHANNEL_0; i <= ADC_CHANNEL_4; i++)
    {
        if (D_SDADC24_WEIGHT_CHANNEL == i)
        {
            p_extended_cfg->p_channel_cfgs[i]   = &g_sdadc24_channel_cfgs;
        }
        else
        {
            p_extended_cfg->p_channel_cfgs[i]   = NULL;
        }
    }
}
/***********************************************************************************************************************
 End of function R_SDADC24_ConfigDisconnectDetection
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_ScanStart
 * Description  : This function calibrate, set offset voltage and start scan.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer
 *                int32_t offset_reg_val
 *                  Value set in the PGAOFS bit of the PGACn register.
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_ScanStart (adc_ctrl_t *p_instance, int32_t offset_reg_val)
{
    fsp_err_t fsp_err;

    fsp_err = R_SDADC_OffsetSet (p_instance, D_SDADC24_WEIGHT_CHANNEL, offset_reg_val);

    if (FSP_SUCCESS == fsp_err)
    {
        R_ICU->IELSR_b[SDADC0_ADI_IRQn].IR      = 0U;
        fsp_err                                 = R_SDADC_ScanStart (p_instance);
    }
    else
    {
        /** Do nothing. */
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function R_SDADC24_ScanStart
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_DisconnectDetectSinglePolarity
 * Description  : This function single polarity disconnect detection.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                bool *p_result
 *                  Pointer to the disconnect detection result storage area.
 *                  true: Connected, false: Disconnected
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_DisconnectDetectSinglePolarity (adc_ctrl_t *p_instance, bool *p_result)
{
    int32_t              sdadc24_ad_value = 0;
    int32_t              sdadc24_ad_value_lasttime = 0;
    volatile uint8_t     i;
    fsp_err_t            fsp_err;

    *p_result   = true;
    fsp_err     = R_SDADC_OffsetSet (p_instance, D_SDADC24_WEIGHT_CHANNEL, E_PGA_SDADC24_OFFSET_0P00);

    if (FSP_SUCCESS == fsp_err)
    {
        R_SDADC0->ADC1_b.PGADISA = 1U;

        if (SDADC_CHANNEL_POLARITY_POSITIVE == R_SDADC0->PGAC_b[D_SDADC24_WEIGHT_CHANNEL].PGAPOL)
        {
            R_SDADC0->ADC1_b.PGADISC = 1U;
        }
        else
        {
            R_SDADC0->ADC1_b.PGADISC = 0U;
        }

        for (i = 0; i < D_SDADC24_DISCONN_NUM; i++)
        {
            /** A/D conversion execution. */
            fsp_err = r_sdadc24_oneshot (p_instance, &sdadc24_ad_value);

            if (FSP_SUCCESS == fsp_err)
            {
                if ((D_SDADC24_OVERFLOW_THRESH <= sdadc24_ad_value) &&
                    (D_SDADC24_OVERFLOW_THRESH <= sdadc24_ad_value_lasttime))
                {
                    *p_result = false;
                    break;
                }
                else
                {
                    /** Do nothing */
                }
            }
            else
            {
                /** Do nothing */
            }
            sdadc24_ad_value_lasttime = sdadc24_ad_value;
        }
        R_SDADC0->ADC1_b.PGADISA = 0U;

        /** Dummy A/D conversion after precharge/discharge.
         *  Over sampling rate is 2048, one-shot time is 2.048[ms].
         *  2.048[ms] * 10 = 20.48[ms]
         */
        for (i = 0; i < D_SDADC24_DISCONN_NUM; i++)
        {
            (void) r_sdadc24_oneshot (p_instance, &sdadc24_ad_value);
        }
    }
    else
    {
        /** Do nothing. */
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function R_SDADC24_DisconnectDetectSinglePolarity
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_DisconnectDetect
 * Description  : This function disconnect detection.
 *                Check for positive polarity and then for negative polarity.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 *                sdadc_scan_cfg_t *p_channel_cfg
 *                  ADC scan configuration structure pointer.
 *                sdadc_channel_cfg_t *p_channel_cfgs
 *                  ADC channel configuration structure pointer.
 *                sdadc_extended_cfg_t *p_extended_cfg
 *                  ADC extended configuration structure pointer.
 *                bool *p_result
 *                  Pointer to the disconnect detection result storage area.
 *                  true: Connected, false: Disconnected
 * Return Value : None
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_DisconnectDetect (adc_ctrl_t *p_instance,               adc_cfg_t *p_cfg,
                                      sdadc_scan_cfg_t *p_channel_cfg,      sdadc_channel_cfg_t *p_channel_cfgs,
                                      sdadc_extended_cfg_t *p_extended_cfg, bool *p_result)
{
    uint8_t     i;
    fsp_err_t   fsp_err = FSP_SUCCESS;

    *p_result = true;

    /** Check positive polarity after negative polarity. */
    for (i = 0; i <= SDADC_CHANNEL_POLARITY_NEGATIVE; i++)
    {
        if (*p_result == true)
        {
            /** SDADC24 setting change and open.
             *  Positive if i is 0, negative if i is 1.
             */
            if (FSP_SUCCESS == fsp_err)
            {
                R_SDADC24_ConfigDisconnectDetection (p_cfg, p_channel_cfg, p_channel_cfgs, p_extended_cfg, i);
                fsp_err = R_SDADC_Open (p_instance, p_cfg);
            }
            else
            {
                /** In case of FSP error, exit the loop. */
                break;
            }

            if (FSP_SUCCESS == fsp_err)
            {
                fsp_err = R_SDADC24_DisconnectDetectSinglePolarity (p_instance, p_result);
            }
            else
            {
                /** In case of FSP error, exit the loop. */
                break;
            }

            if (FSP_SUCCESS == fsp_err)
            {
                fsp_err = R_SDADC_Close (p_instance);
            }
            else
            {
                /** In case of FSP error, exit the loop. */
                break;
            }
        }
        else
        {
            /** Do nothing. */
        }
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function R_SDADC24_DisconnectDetect
  **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_GetGain
 * Description  : This function get PGA gain setting.
 * Arguments    : st_gain_setting_t *p_gain
 *                  Gain setting structure pointer.
 *                adc_channel_t adc_channel
 *                  Target channel.
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_GetGain (st_gain_setting_t *p_gain, adc_channel_t adc_channel)
{
    uint32_t pgac_pgagc;

    pgac_pgagc      = R_SDADC0->PGAC_b[adc_channel].PGAGC;
    p_gain->gset1   = gs_gain_list[pgac_pgagc][0];
    p_gain->gset2   = gs_gain_list[pgac_pgagc][1];
    p_gain->gtotal  = gs_gain_list[pgac_pgagc][2];
}
/***********************************************************************************************************************
 End of function R_SDADC24_GetGain
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_GetOffset
 * Description  : This function get offset voltage (converted value to A/D value).
 * Arguments    : adc_channel_t adc_channel
 *                  Target channel.
 * Return Value : int32_t
 *                  Offset voltage. (converted value to A/D value)
 **********************************************************************************************************************/
int32_t R_SDADC24_GetOffset (adc_channel_t adc_channel)
{
    uint32_t pgac_pgaofs;

    pgac_pgaofs = R_SDADC0->PGAC_b[adc_channel].PGAOFS;

    return gs_offset_list[pgac_pgaofs];
}
/***********************************************************************************************************************
 End of function R_SDADC24_GetOffset
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: r_sdadc24_oneshot
 * Description  : This function A/D conversion is executed once.
 *                The A/D conversion mode of SDADC24 must be single scan.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                int32_t *p_ad_value
 *                  A/D conversion value storage area pointer.
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
static fsp_err_t r_sdadc24_oneshot (adc_ctrl_t *p_instance, int32_t *p_ad_value)
{
    fsp_err_t       fsp_err;
    adc_status_t    adc_status;

    R_ICU->IELSR_b[SDADC0_ADI_IRQn].IR      = 0U;
    fsp_err                                 = R_SDADC_ScanStart (p_instance);

    /** Wait for calibration complete. */
    if (FSP_SUCCESS == fsp_err)
    {
        adc_status.state = ADC_STATE_SCAN_IN_PROGRESS;

        while (ADC_STATE_SCAN_IN_PROGRESS == adc_status.state)
        {
            fsp_err = R_SDADC_StatusGet (p_instance, &adc_status);
            /** Handle error */
            if (FSP_SUCCESS != fsp_err)
            {
                break;
            }
            else
            {
                /** Do nothing */
            }
        }

        if (FSP_SUCCESS == fsp_err)
        {
            *p_ad_value = (int32_t) g_sdadc24_ad_value;

            /** A/D value does not over flow. */
            if (*p_ad_value < D_SDADC24_OVERFLOW_THRESH)
            {
                /** Sign extension. */
                *p_ad_value = (*p_ad_value << 8U) >> 8U;
            }
            else
            {
                /** Do nothing */
            }
        }
        else
        {
            /** Do nothing. */
        }
    }
    else
    {
        /** Do nothing. */
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function r_sdadc24_oneshot
 **********************************************************************************************************************/

