/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_usb_control_api.c
* Version      : 1.0
* Description  : SDADC24 control module
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 30.04.2020 1.00     First Release
***********************************************************************************************************************/

/***********************************************************************************************************************
 Includes <System Includes> , “Project Includes”
 **********************************************************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "board_cfg.h"
#include "hal_data.h"
#include "r_usb_control_api.h"

/***********************************************************************************************************************
 Macro definitions
 **********************************************************************************************************************/

/***********************************************************************************************************************
 Local Typedef definitions
 **********************************************************************************************************************/

/***********************************************************************************************************************
 Exported global variables (to be accessed by other files)
 **********************************************************************************************************************/
const usb_descriptor_t  g_usb_descriptor =
{
    g_apl_device,                                       /* Pointer to the device descriptor                           */
    g_apl_configuration,                                /* Pointer to the configuration descriptor for Full-speed     */
    g_apl_hs_configuration,                             /* Pointer to the configuration descriptor for Hi-speed       */
    g_apl_qualifier_descriptor,                         /* Pointer to the qualifier descriptor                        */
    g_apl_string_table,                                 /* Pointer to the string descriptor table                     */
    D_NUM_STRING_DESCRIPTOR
};

/***********************************************************************************************************************
 Private (static) variables and functions
 **********************************************************************************************************************/
static bool s_usb_write_complete = true;                /* USB write complete event flag                              */

/***********************************************************************************************************************
 * Function Name: R_USBCTRL_EventPoll
 * Description  : This function processing according to USB event.
 * Arguments    : st_usbctrl_ctrl_t *p_usbctrl_ctrl
 *                  USB control structure pointer.
 * Return Value : None
 **********************************************************************************************************************/
fsp_err_t R_USBCTRL_EventPoll (st_usbctrl_ctrl_t * const p_usbctrl_ctrl)
{
    fsp_err_t fsp_err;

    /** Obtain USB related events */
    fsp_err = R_USB_EventGet (p_usbctrl_ctrl->p_ctrl, p_usbctrl_ctrl->p_status);

    /** USB event received by R_USB_EventGet */
    if (FSP_SUCCESS == fsp_err)
    {
        switch (*p_usbctrl_ctrl->p_status)
        {
            case USB_STATUS_CONFIGURED:
            {
                /** USB data read */
                fsp_err = R_USB_Read (p_usbctrl_ctrl->p_ctrl,
                                      p_usbctrl_ctrl->p_readbuf,
                                      D_USB_EP_PACKET_SIZE, USB_CLASS_PCDC);
                break;
            }

            case USB_STATUS_WRITE_COMPLETE:
            {
                /** Set write complete flag */
                s_usb_write_complete = true;
                /** USB data read  */
                R_USB_Read (p_usbctrl_ctrl->p_ctrl,
                                      p_usbctrl_ctrl->p_readbuf,
                                      D_USB_EP_PACKET_SIZE, USB_CLASS_PCDC);
                break;
            }

            case USB_STATUS_READ_COMPLETE:
            {
                /** USB data read  */
                fsp_err = R_USB_Read (p_usbctrl_ctrl->p_ctrl,
                                      p_usbctrl_ctrl->p_readbuf,
                                      p_usbctrl_ctrl->readbuf_size, USB_CLASS_PCDC);
                break;
            }

            /** Receive class request */
            case USB_STATUS_REQUEST :
            {
                fsp_err = R_USB_SetupGet(p_usbctrl_ctrl->p_ctrl, p_usbctrl_ctrl->p_setup);

                /** Check for the specific CDC class request IDs */
                if (FSP_SUCCESS == fsp_err)
                {
                    if (USB_PCDC_SET_LINE_CODING == (p_usbctrl_ctrl->p_setup->request_type & USB_BREQUEST))
                    {
                        fsp_err = R_USB_PeriControlDataGet (p_usbctrl_ctrl->p_ctrl,
                                                            (uint8_t *) p_usbctrl_ctrl->p_linecoding,
                                                            D_LINE_CODING_LENGTH);
                    }
                    else if (USB_PCDC_GET_LINE_CODING == (p_usbctrl_ctrl->p_setup->request_type & USB_BREQUEST))
                    {
                        fsp_err = R_USB_PeriControlDataSet (p_usbctrl_ctrl->p_ctrl,
                                                            (uint8_t *) p_usbctrl_ctrl->p_linecoding,
                                                            D_LINE_CODING_LENGTH);
                    }
                    else
                    {
                        fsp_err = R_USB_PeriControlStatusSet (p_usbctrl_ctrl->p_ctrl, USB_SETUP_STATUS_ACK);
                    }
                }
                break;
            }

            case USB_STATUS_DETACH:
            case USB_STATUS_SUSPEND:
            case USB_STATUS_RESUME:
            {
                break;
            }

            default:
            {
                break;
            }
        }
    }
    else
    {
        /** Do nothing */
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function R_USBCTRL_EventPoll
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_USBCTRL_Write
 * Description  : This function start sending data.
 * Arguments    : st_usbctrl_ctrl_t *p_usbctrl_ctrl
 *                  USB control structure pointer
 *                uint8_t *p_buf
 *                  Send data pointer.
 *                uint32_t size
 *                  Size of send data.
 *                uint8_t destination
 *                  USB class type.
 * Return Value : None
 **********************************************************************************************************************/
fsp_err_t R_USBCTRL_Write (st_usbctrl_ctrl_t * const p_usbctrl_ctrl, uint8_t const * const p_buf,
                           uint32_t size                           , uint8_t destination)
{
    fsp_err_t fsp_err;

    fsp_err = R_USB_Write (p_usbctrl_ctrl->p_ctrl, p_buf, size, destination);

    if (FSP_SUCCESS == fsp_err)
    {
        s_usb_write_complete = false;
    }
    else
    {
        /** Do nothing. */
    }

    return fsp_err;
}
/***********************************************************************************************************************
 End of function R_USBCTRL_Write
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_USBCTRL_GetWriteStatus
 * Description  : This function get USB write status.
 * Arguments    : None
 * Return Value : bool
 *                  true:Complete, false:Incomplete
 **********************************************************************************************************************/
bool R_USBCTRL_GetWriteStatus (void)
{
    return s_usb_write_complete;
}
/***********************************************************************************************************************
 End of function R_USBCTRL_GetWriteStatus
 **********************************************************************************************************************/



