/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * File Name    : r_weight_calculate_api.c
 * Version      : 1.0
 * Description  : weight calculate source file
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 30.04.2020 1.00     First Release
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "r_sdadc24_control_api.h"
#include "r_weight_calculate_api.h"

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Typedef definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Private (static) variables and functions
 *********************************************************************************************************************/
static float    gs_weight_zero                                = 0.0F;
static int32_t  gs_previous_buf[D_WEIGHT_STABILITY_CHECK_NUM] = {0x7FFFFFFF};
static uint8_t  gs_previous_index                             = 0U;

/**********************************************************************************************************************
 * Function Name: r_weight_stability_check_init
 * Description  : This function initialize variables for stable state determination processing.
 * Arguments    : void
 * Return Value : void
 *********************************************************************************************************************/
static void r_weight_stability_check_init (void);

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_SetZeroAdjustWeight
 * Description  : This function set the zero adjustment weight.
 * Arguments    : float weight -
 *                  zero adjustment weight[g].
 * Return Value : float
 *                  A/D value (correct value).
 *********************************************************************************************************************/
void R_WEIGHT_SetZeroAdjustWeight (float weight)
{
    gs_weight_zero = weight;
}
/**********************************************************************************************************************
 End of function R_WEIGHT_SetZeroAdjustWeight
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_GetZeroAdjustWeight
 * Description  : This function get the zero adjustment weight.
 * Arguments    : float weight -
 *                  zero adjustment weight[g].
 * Return Value : float
 *                  A/D value (correct value).
 *********************************************************************************************************************/
float R_WEIGHT_GetZeroAdjustWeight (void)
{
    return gs_weight_zero;
}
/**********************************************************************************************************************
 End of function R_WEIGHT_GetZeroAdjustWeight
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_ADValueCorrect
 * Description  : This function subtract gain and offset from A/D value.
 * Arguments    : float ad_value -
 *                  A/D value.
 *              : st_gain_setting_t *p_gain -
 *                  Gain setting value structure pointer.
 *              : int32_t offset -
 *                  Offset voltage (converted value to A/D value).
 * Return Value : float
 *                  A/D value (correct value).
 *********************************************************************************************************************/
float R_WEIGHT_ADValueCorrect (float ad_value, st_gain_setting_t *p_gain, int32_t offset)
{
    int32_t offset_calc;

    offset_calc = (offset * p_gain->gset2);
    return ((ad_value - (float) offset_calc) / (float) p_gain->gtotal);
}
/**********************************************************************************************************************
 End of function R_WEIGHT_ADValueCorrect
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_Calculate
 * Description  : This function inputs the A/D value and returns the weight.
 * Arguments    : float ad_value -
 *                    A/D value.
 *              : float coef_a -
 *                    weight conversion coefficient A.
 *              : float coef_b -
 *                    weight conversion coefficient B.
 * Return Value : float
 *                    weight[g].
 *********************************************************************************************************************/
float R_WEIGHT_Calculate (float ad_value, float coef_a, float coef_b)
{
    return ((ad_value * coef_a) + coef_b);
}
/**********************************************************************************************************************
 End of function R_WEIGHT_Calculate
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: r_weight_stability_check_init
 * Description  : This function initialize variables for stable state determination processing.
 * Arguments    : void
 * Return Value : void
 *********************************************************************************************************************/
static void r_weight_stability_check_init (void)
{
    uint8_t i;

    gs_previous_index = 0U;
    for (i = 0; i < D_WEIGHT_STABILITY_CHECK_NUM; i++)
    {
        gs_previous_buf[i] = 0x7FFFFFFF;
    }
}
/**********************************************************************************************************************
 End of function r_weight_stability_check_init
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_StabilityCheck
 * Description  : This function determine the stable state of the weight measurement value.
 * Arguments    : float weight -
 *                    weight[g].
 *                bool *p_clear_request
 *                    Request to clear the stability check variables.
 *                    If true, clear and then store data in the buffer, return the request flag as false.
 *                    true: Clear, false: Do not clear
 * Return Value : bool
 *                    true: Stable, false: Not stable
 *********************************************************************************************************************/
bool R_WEIGHT_StabilityCheck (float weight, bool *p_clear_request)
{
    int32_t upper_limit;
    int32_t lower_limit;
    int32_t difference;
    uint8_t check_index     = 0U;
    uint8_t reference_index = 0U;
    bool    ret             = true;

    /** Clear the stability check variables. */
    if (true == *p_clear_request)
    {
        r_weight_stability_check_init ();
        *p_clear_request = false;
    }
    else
    {
        /** Do nothing */
    }

    gs_previous_buf[gs_previous_index]  = (int32_t) (weight * 1000);      /* weight[g] -> weight[mg] */
    gs_previous_index++;

    if (D_WEIGHT_STABILITY_CHECK_NUM <= gs_previous_index)
    {
        gs_previous_index = 0U;
    }

    upper_limit = (int32_t) D_WEIGHT_STABILITY_CHECK_VALUE;
    lower_limit = -1 * (int32_t) D_WEIGHT_STABILITY_CHECK_VALUE;

    for (reference_index = 0U; reference_index < D_WEIGHT_STABILITY_CHECK_NUM; reference_index++)
    {
        for (check_index = 0U; check_index < D_WEIGHT_STABILITY_CHECK_NUM; check_index++)
        {
            if (reference_index != check_index)
            {
                difference = gs_previous_buf[reference_index] - gs_previous_buf[check_index];

                if ((upper_limit < difference) || (lower_limit > difference))
                {
                    ret = false;
                    break;
                }
                else
                {
                    /** Do nothing */
                }
            }
            else
            {
                /** Do nothing */
            }
        }

        if (false == ret)
        {
            break;
        }
    }

    return ret;
}
/**********************************************************************************************************************
 End of function R_WEIGHT_StabilityCheck
 *********************************************************************************************************************/
