/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * File Name    : r_weight_calculate_api.h
 * Version      : 1.0
 * Description  : Weight calculate api header file
 *********************************************************************************************************************/
/**********************************************************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 30.04.2020 1.00     First Release
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/

#include <stdint.h>
#include <stdbool.h>

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/
#ifndef R_WEIGHT_CALCULATE_API_H
#define R_WEIGHT_CALCULATE_API_H

#define D_LOADCELL_MAX_WEIGHT           (500.0F)
#define D_LOADCELL_MIN_WEIGHT           (-500.0F)
#define D_LOADCELL_OVERLOAD_PERCENT     (125.0F)
#define D_LOADCELL_COEFFICIENT_A        (0.000987142F)
#define D_LOADCELL_COEFFICIENT_B        (-50.10420444F)
#define D_LOADCELL_MAX_OVERLOAD         (D_LOADCELL_MAX_WEIGHT * (D_LOADCELL_OVERLOAD_PERCENT / 100.0F))
#define D_LOADCELL_MIN_OVERLOAD         (D_LOADCELL_MIN_WEIGHT * (D_LOADCELL_OVERLOAD_PERCENT / 100.0F))

/** Used to judge the stable state of the weight measurement value.
 *  Value for determining the stability of weight measurement[mg].
 *  30 -> 30[mg]
 */
#define D_WEIGHT_STABILITY_CHECK_VALUE  (30U)

/** The number of times the measured value is considered stable.
 *  If the weight difference continues for D_WEIGHT_STABILITY_CHECK_NUM times and is within the range of
 *  D_WEIGHT_STABILITY_CHECK_VALUE, it will be a stable state.
 * */
#define D_WEIGHT_STABILITY_CHECK_NUM    (3U)

/**********************************************************************************************************************
 Typedef definitions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global variables
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Exported global functions
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_SetZeroAdjustWeight
 * Description  : This function set the zero adjustment weight.
 * Arguments    : float weight -
 *                  zero adjustment weight[g].
 * Return Value : float
 *                  A/D value (correct value).
 *********************************************************************************************************************/
void R_WEIGHT_SetZeroAdjustWeight (float weight);

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_GetZeroAdjustWeight
 * Description  : This function get the zero adjustment weight.
 * Arguments    : float weight -
 *                  zero adjustment weight[g].
 * Return Value : float
 *                  A/D value (correct value).
 *********************************************************************************************************************/
float R_WEIGHT_GetZeroAdjustWeight (void);

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_ADValueCorrect
 * Description  : This function subtract gain and offset from A/D value.
 * Arguments    : float ad_value -
 *                  A/D value.
 *              : st_gain_setting_t *p_gain -
 *                  Gain setting value structure pointer.
 *              : int32_t offset -
 *                  Offset voltage (converted value to A/D value).
 * Return Value : float
 *                  A/D value (correct value).
 *********************************************************************************************************************/
float R_WEIGHT_ADValueCorrect (float ad_value, st_gain_setting_t *p_gain, int32_t offset);

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_Calculate
 * Description  : This function inputs the A/D value and returns the weight.
 * Arguments    : float ad_value -
 *                    A/D value.
 *              : float coef_a -
 *                    weight conversion coefficient A.
 *              : float coef_b -
 *                    weight conversion coefficient B.
 * Return Value : float
 *                    weight[g].
 *********************************************************************************************************************/
float R_WEIGHT_Calculate (float ad_value, float coef_a, float coef_b);

/**********************************************************************************************************************
 * Function Name: R_WEIGHT_StabilityCheck
 * Description  : This function determine the stable state of the weight measurement value.
 * Arguments    : float weight -
 *                    weight[g].
 *                bool *p_clear_request
 *                    Request to clear the stability check variables.
 *                    If true, clear and then store data in the buffer, return the request flag as false.
 *                    true: Clear, false: Do not clear
 * Return Value : bool
 *                    true: Stable, false: Not stable
 *********************************************************************************************************************/
bool R_WEIGHT_StabilityCheck (float weight, bool *p_clear_request);

#endif /* R_WEIGHT_CALCULATE_API_H */
